/* Copyright (C) 2019 Intel Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Lionel Landwerlin <lionel.g.landwerlin@intel.com>
 */

#pragma once
#include "core_validation.h"

class CommandCounter : public ValidationObject {
  public:
    CommandCounter(CoreChecks *coreChecks) : coreChecks(coreChecks) {}
    virtual ~CommandCounter() {}

    virtual write_lock_guard_t write_lock() { return coreChecks->write_lock(); }

#include "command_counter_helper.h"

  private:
    CoreChecks *coreChecks;
};
